#include "Registry.h"

createFileSingleton( Registry );

bool operator<( const CRegEntry & re1, const CRegEntry & re2 )
{
	return re1.m_qstrKey < re2.m_qstrKey;
}

bool Registry::initialize()
{
	bool bRet = readRegistry();

	if( bRet )
		m_qlRegistry = m_qlRegistrySaved;

	return bRet;
}

bool Registry::save()
{
	bool bRet = saveRegistry();

	if( bRet )
		m_qlRegistrySaved = m_qlRegistry;

	return bRet;
}

const QRegEntryList & Registry::getEntries()
{
	return m_qlRegistry;
}

const QRegEntryList & Registry::getEntriesSaved()
{
	return m_qlRegistrySaved;
}

/* if an entry already exists eretType is ignored */
bool Registry::setEntry( const QString & qstrKey, const QString & qstrVal, ERegEntryType eretType )
{
	/* check if "key" got ':' in the middle */
	QRegExp rx( ".+:.+" );
	if( ! rx.exactMatch( qstrKey ) )
		return false;

	int i;
	for( i = 0; i < m_qlRegistry.size(); ++i )
		if( m_qlRegistry.at( i ).m_qstrKey == qstrKey )
			break;

	/* nothing found, add new */
	if( i == m_qlRegistry.size() )
	{
		CRegEntry re;
		re.m_qstrKey = qstrKey;
		re.m_qstrVal = qstrVal;
		re.m_eretType = eretType;

		m_qlRegistry.append( re );
		qSort( m_qlRegistry );
		return true;
	}

	m_qlRegistry[ i ].m_qstrVal = qstrVal;
	return true;
}

bool Registry::delEntry( const QString & qstrKey )
{
	for( int i = 0; i < m_qlRegistry.size(); i++ )
		if( m_qlRegistry.at( i ).m_qstrKey == qstrKey )
		{
			m_qlRegistry.removeAt( i );
			return true;
		}

	return false;
}

int Registry::findEntryIndex( const QString & qstrKey ) const
{
	for( int i = 0; i < m_qlRegistry.size(); ++i )
		if( m_qlRegistry.at( i ).m_qstrKey == qstrKey )
			return i;

	return -1;
}

/**************************** VIDEO *******************************/

bool Registry::setResolution( const QString & qstrRes )
{
	QRegExp rx( "[0-9]+x[0-9]+" );
	if( ! rx.exactMatch( qstrRes ) )
		return false;

	int nDepth = getColorDepth();
	if( nDepth <= 0 )
		nDepth = 16;

	QString qstrVal = QString( "OGL -(%1)x%2 bit" ).arg( qstrRes ).arg( nDepth );

	return setEntry( "Default:VideocardFs2open", qstrVal, eretString );
}

bool Registry::setColorDepth( const int & nDepth )
{
	if( nDepth < 0 )
		return false;

	QString qstrRes = getResolution();
	if( qstrRes.isEmpty() )
		qstrRes = "640x480";

	QString qstrVal = QString( "OGL -(%1)x%2 bit" ).arg( qstrRes ).arg( nDepth );

	return setEntry( "Default:VideocardFs2open", qstrVal, eretString );
}

bool Registry::setTextureFilter( const int & nFilter )
{
	if( nFilter < 0 )
		return false;

	return setEntry( "Default:TextureFilter", QString( "%1" ).arg( nFilter ), eretDWORD );
}

bool Registry::setAnisotropic( const float & fAF )
{
	if( fAF < 0 )
		return false;

	return setEntry( "Default:OGL_AnisotropicFilter", QString( "%1" ).arg( fAF, 0, 'f', 1 ), eretString );
}

bool Registry::setAntialiasing( const int & nAA )
{
	if( nAA < 0 )
		return false;

	return setEntry( "Default:OGL_AntiAliasSamples", QString( "%1" ).arg( nAA ), eretDWORD );
}

bool Registry::setComputerSpeed( const int & nSpeed )
{
	if( nSpeed < 0 )
		return false;

	return setEntry( "Default:ComputerSpeed", QString( "%1" ).arg( nSpeed ), eretDWORD );
}

QString Registry::getResolution() const
{
	int i = findEntryIndex( "Default:VideocardFs2open" );

	/* not found */
	if( i < 0 )
		return QString();

	QRegExp rx( "[()]" );
	/* this should return empty string on false */
	return m_qlRegistry.at( i ).m_qstrVal.section( rx, 1, 1 );
}

int Registry::getColorDepth() const
{
	int i = findEntryIndex( "Default:VideocardFs2open" );

	/* not found */
	if( i < 0 )
		return -1;

	QRegExp rx( "[\\sx]" );
	bool bOk;
	int nDepth = m_qlRegistry.at( i ).m_qstrVal.section( rx, 3, 3 ).toInt( &bOk );
	if( bOk )
		return nDepth;

	return -1;
}

int Registry::getTextureFilter() const
{
	int i = findEntryIndex( "Default:TextureFilter" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nFilter = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nFilter;

	return -1.0f;
}

float Registry::getAnisotropic() const
{
	int i = findEntryIndex( "Default:OGL_AnisotropicFilter" );

	/* not found */
	if( i < 0 )
		return -1.0f;

	bool bOk;
	float fAF = m_qlRegistry.at( i ).m_qstrVal.toFloat( &bOk );
	if( bOk )
		return fAF;

	return -1.0f;
}

int Registry::getAntialiasing() const
{
	int i = findEntryIndex( "Default:OGL_AntiAliasSamples" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nAA = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nAA;

	return -1;
}

int Registry::getComputerSpeed() const
{
	int i = findEntryIndex( "Default:ComputerSpeed" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nSpeed = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nSpeed;

	return -1;
}

/**************************** AUDIO/JOY *******************************/

bool Registry::setAudioDev( const QString & qstrAudio )
{
	if( qstrAudio.isEmpty() )
		return false;

	return setEntry( "Default:SoundDeviceOAL", qstrAudio, eretString );
}

bool Registry::setJoystickDev( const int & nJoy )
{
	if( nJoy < 0 )
		return false;

	return setEntry( "Default:CurrentJoystick", QString( "%1" ).arg( nJoy ), eretDWORD );
}

bool Registry::setForceFeedback( const int & nFF )
{
	if( nFF < 0 )
		return false;

	return setEntry( "Default:EnableJoystickFF", QString( "%1").arg( nFF ), eretDWORD );
}

bool Registry::setHitEffect( const int & nHE )
{
	if( nHE < 0 )
		return false;

	return setEntry( "Default:EnableHitEffect", QString( "%1").arg( nHE ), eretDWORD );
}

QString Registry::getAudioDev() const
{
	int i = findEntryIndex( "Default:SoundDeviceOAL" );

	/* not found */
	if( i < 0 )
		return QString();

	return m_qlRegistry.at( i ).m_qstrVal;
}

int Registry::getJoystickDev() const
{
	int i = findEntryIndex( "Default:CurrentJoystick" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nJoy = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nJoy;

	return -1;
}

int Registry::getForceFeedback() const
{
	int i = findEntryIndex( "Default:EnableJoystickFF" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nFF = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nFF;

	return -1;
}

int Registry::getHitEffect() const
{
	int i = findEntryIndex( "Default:EnableHitEffect" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nHE = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nHE;

	return -1;
}

/**************************** SPEECH *******************************/

bool Registry::setVoice( const int & nVoice )
{
	if( nVoice < 0 )
		return false;

	return setEntry( "Default:SpeechVoice", QString( "%1" ).arg( nVoice ), eretDWORD );
}

bool Registry::setVolume( const int & nVolume )
{
	if( nVolume < 0 )
		return false;

	return setEntry( "Default:SpeechVolume", QString( "%1" ).arg( nVolume ), eretDWORD );
}

bool Registry::setTechRoom( const int & nTechRoom )
{
	if( nTechRoom < 0 )
		return false;

	return setEntry( "Default:SpeechTechRoom", QString( "%1" ).arg( nTechRoom ), eretDWORD );
}

bool Registry::setBriefings( const int & nBriefings )
{
	if( nBriefings < 0 )
		return false;

	return setEntry( "Default:SpeechBriefings", QString( "%1" ).arg( nBriefings ), eretDWORD );
}

bool Registry::setIngame( const int & nIngame )
{
	if( nIngame < 0 )
		return false;

	return setEntry( "Default:SpeechIngame", QString( "%1" ).arg( nIngame ), eretDWORD );
}

int Registry::getVoice() const
{
	int i = findEntryIndex( "Default:SpeechVoice" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	QString q = m_qlRegistry.at( i ).m_qstrVal;
	int nVoice = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nVoice;

	return -1;
}

int Registry::getVolume() const
{
	int i = findEntryIndex( "Default:SpeechVolume" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nVolume = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nVolume;

	return -1;
}

int Registry::getTechRoom() const
{
	int i = findEntryIndex( "Default:SpeechTechRoom" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nTechRoom = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nTechRoom;

	return -1;
}

int Registry::getBriefings() const
{
	int i = findEntryIndex( "Default:SpeechBriefings" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nBriefings = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nBriefings;

	return -1;
}

int Registry::getIngame() const
{
	int i = findEntryIndex( "Default:SpeechIngame" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nIngame = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nIngame;

	return -1;
}

/**************************** NETWORK *******************************/

bool Registry::setNetworkConnection( const int & nNet )
{
	if( nNet < 0 )
		return false;

	QString qstrNet;
	switch( nNet )
	{
	case 0:
		qstrNet = "None";
		break;
	case 1:
		qstrNet = "Dialup";
		break;
	case 2:
		qstrNet = "LAN";
		break;
	default:
		return false;
	}

	return setEntry( "Default:NetworkConnection", qstrNet, eretString );
}

bool Registry::setConnectionSpeed( const int & nSpeed )
{
	if( nSpeed < 0 )
		return false;

	QString qstrSpeed;
	switch( nSpeed )
	{
	case 0:
		qstrSpeed = "None";
		break;
	case 1:
		qstrSpeed = "Slow";
		break;
	case 2:
		qstrSpeed = "56K";
		break;
	case 3:
		qstrSpeed = "ISDN";
		break;
	case 4:
		qstrSpeed = "Cable";
		break;
	case 5:
		qstrSpeed = "Fast";
		break;
	default:
		return false;
	}

	return setEntry( "Default:ConnectionSpeed", qstrSpeed, eretString );
}

bool Registry::setLocalPort( const int & nPort )
{
	if( nPort < 0 )
		return false;

	/* be nice and remove entry altogether */
	if( nPort == 0 )
		return delEntry( "Default:ForcePort" );

	return setEntry( "Default:ForcePort", QString( "%1" ).arg( nPort ), eretDWORD );
}

bool Registry::setIPAddress( const QString & qstrIP )
{
	/* be nice and remove entry altogether */
	if( qstrIP == "" )
		return delEntry( "Network:CustomIP" );

	return setEntry( "Network:CustomIP", qstrIP, eretString );
}

int Registry::getNetworkConnection() const
{
	int i = findEntryIndex( "Default:NetworkConnection" );

	/* not found */
	if( i < 0 )
		return -1;

	QString qstrNet = m_qlRegistry.at( i ).m_qstrVal;
	if( qstrNet == "None" )
		return 0;
	else if( qstrNet == "Dialup" )
		return 1;
	else if( qstrNet == "LAN" )
		return 2;

	return -1;
}

int Registry::getConnectionSpeed() const
{
	int i = findEntryIndex( "Default:ConnectionSpeed" );

	/* not found */
	if( i < 0 )
		return -1;

	QString qstrSpeed = m_qlRegistry.at( i ).m_qstrVal;
	if( qstrSpeed == "None" )
		return 0;
	else if( qstrSpeed == "Slow" )
		return 1;
	else if( qstrSpeed == "56K" )
		return 2;
	else if( qstrSpeed == "ISDN" )
		return 3;
	else if( qstrSpeed == "Cable" )
		return 4;
	else if( qstrSpeed == "Fast" )
		return 5;

	return -1;
}

int Registry::getLocalPort() const
{
	int i = findEntryIndex( "Default:ForcePort" );

	/* not found */
	if( i < 0 )
		return -1;

	bool bOk;
	int nPort = m_qlRegistry.at( i ).m_qstrVal.toInt( &bOk );
	if( bOk )
		return nPort;

	return -1;
}

QString Registry::getIPAddress() const
{
	int i = findEntryIndex( "Network:CustomIP" );

	/* not found */
	if( i < 0 )
		return QString();

	return m_qlRegistry.at( i ).m_qstrVal;
}
