#ifndef REGISTRY_H
#define REGISTRY_H

#include <QtCore>

#include "Singleton.h"

#define REGISTRY ( Registry::getSingleton() )

enum ERegEntryType
{
	eretString = 0,
	eretDWORD,

	eretTotal
};

struct CRegEntry
{
	QString m_qstrKey;
	QString m_qstrVal;
	ERegEntryType m_eretType;
};

typedef QList< CRegEntry > QRegEntryList;
bool operator<( const CRegEntry & re1, const CRegEntry & re2 );

class Registry : public Singleton < Registry >
{
private:
	QRegEntryList m_qlRegistry;
	QRegEntryList m_qlRegistrySaved;

	bool readRegistry();
	bool saveRegistry();
	bool setEntry( const QString&, const QString&, ERegEntryType );
	bool delEntry( const QString& );
	int findEntryIndex( const QString & qstrKey ) const;

public:
	/* main processing functions */
	bool initialize();
	bool save();

	const QRegEntryList & getEntries();
	const QRegEntryList & getEntriesSaved();

	/*
	 * lot's of helper function, theoreticaly they are bogus and
	 * setEntry/getEntry would be enough, but they are convienient
	 */

	/* video */
	bool setResolution( const QString& );
	bool setColorDepth( const int& );
	bool setTextureFilter( const int& );
	bool setAnisotropic( const float& );
	bool setAntialiasing( const int& );
	bool setComputerSpeed( const int& );

	QString getResolution() const;
	int getColorDepth() const;
	int getTextureFilter() const;
	float getAnisotropic() const;
	int getAntialiasing() const;
	int getComputerSpeed() const;

	/* audio/joy */
	bool setAudioDev( const QString& );
	bool setJoystickDev( const int& );
	bool setForceFeedback( const int& );
	bool setHitEffect( const int& );

	QString getAudioDev() const;
	int getJoystickDev() const;
	int getForceFeedback() const;
	int getHitEffect() const;

	/* speech */
	bool setVoice( const int& );
	bool setVolume( const int& );
	bool setTechRoom( const int& );
	bool setBriefings( const int& );
	bool setIngame( const int& );

	int getVoice() const;
	int getVolume() const;
	int getTechRoom() const;
	int getBriefings() const;
	int getIngame() const;

	/* network */
	bool setNetworkConnection( const int& );
	bool setConnectionSpeed( const int& );
	bool setLocalPort( const int& );
	bool setIPAddress( const QString& );

	int getNetworkConnection() const;
	int getConnectionSpeed() const;
	int getLocalPort() const;
	QString getIPAddress() const;
};

#endif
