#ifndef SETTINGS_H
#define SETTINGS_h

#include <QtCore>

#include "Singleton.h"

#define SETTINGS ( Settings::getSingleton() )

struct CFlag
{
	QString m_qstrFlag;
	QString m_qstrDesc;
	int m_nEasyFlagsOn;
	int m_nEasyFlagsOff;
	QString m_qstrType;
	QString m_qstrUrl;
	bool m_bState;
};

typedef QList< CFlag > QFlagList;

class Settings : public Singleton < Settings >
{
private:
	bool parseFlagList();

	/* path to game binary */
	QString m_qstrExeFilePath;
	/* custom flags */
	QString m_qstrCustomFlags;
	/* flags */
	QFlagList m_qlFlags;
	/* flag categories */
	QStringList m_qstrlFlagTypes;
	/* easy setups */
	QStringList m_qstrlEasyFlagSets;

	/* mod name */
	QString m_qstrModName;
	QString m_qstrModFullName;
	/* mod info */
	QString m_qstrModImage, m_qstrModInfo, m_qstrModWebsite, m_qstrModForum;
	/* list of additional mods */
	QStringList m_qstrlModsPrimary, m_qstrlModsSecondary;

public:
	/* main processing functions */
	bool initialize();
	bool save() const;

	bool setExeFilePath( const QString& );
	bool setModName( const QString& );
	void setFlag( const QString&, bool, bool = false );
	void setCustomFlags( const QString& );
	void setEasyTypeSet( int );
	QString generateCmdLine( bool = true, bool = true ) const;

	/* dumb getters */
	const QString & getExeFilePath() const;
	const QString & getCustomFlags() const;
	const QFlagList & getFlags() const;
	const QStringList & getFlagTypes() const;
	const QStringList & getEasyFlagSets() const;

	const QString & getModName() const;
	const QString & getModFullName() const;
	const QString & getModImage() const;
	const QString & getModInfo() const;
	const QString & getModWebsite() const;
	const QString & getModForum() const;
};

#endif
