#include "MainWindow.h"
#include "ui_MainWindow.h"

#include <QtGui>

#include "Settings.h"
#include "Registry.h"
#include "System.h"
#include "FilePaths.h"
#include "GameInfo.h"


MainWindow::MainWindow( QWidget *parent ) :
	QMainWindow( parent ),
	ui( new Ui::MainWindow )
{
	ui->setupUi( this );

	setFixedSize( System::getSize() );
	setWindowTitle( GameInfo::getWindowTitle() + QString( " " ) + GameInfo::getLauncherVersion() );

	QButtonGroup * qbgTabsButtons = new QButtonGroup( this );
	qbgTabsButtons -> addButton( ui->qrbWelcome, 0 );
	qbgTabsButtons -> addButton( ui->qrbFeatures, 1 );
	qbgTabsButtons -> addButton( ui->qrbMod, 2 );
	qbgTabsButtons -> addButton( ui->qrbVideo, 3 );
	qbgTabsButtons -> addButton( ui->qrbAudioJoy, 4 );
	qbgTabsButtons -> addButton( ui->qrbSpeech, 5 );
	qbgTabsButtons -> addButton( ui->qrbNetwork, 6 );
	qbgTabsButtons -> addButton( ui->qrbRegistry, 7 );

	connect( qbgTabsButtons, SIGNAL( buttonClicked( int ) ), ui->qswTabs, SLOT( setCurrentIndex( int ) ) );
	ui -> qrbWelcome -> click();

	// tabs that can update flags (but TabFeatures)
	connect( ui -> qwTabMod, SIGNAL( setFlagsChanged() ), this, SLOT( flagsChanged() ) );

	// tabs that can update registry (but TabRegistry)
	connect( ui -> qwTabVideo, SIGNAL( setRegistryChanged() ), this, SLOT( registryChanged() ) );
	connect( ui -> qwTabAudioJoy, SIGNAL( setRegistryChanged() ), this, SLOT( registryChanged() ) );
	connect( ui -> qwTabSpeech, SIGNAL( setRegistryChanged() ), this, SLOT( registryChanged() ) );
	connect( ui -> qwTabNetwork, SIGNAL( setRegistryChanged() ), this, SLOT( registryChanged() ) );

	updateContent();
}

MainWindow::~MainWindow()
{
	delete ui;
}

void MainWindow::updateContent()
{
	//ui -> qleBinaryPath -> setText( QFileInfo( SETTINGS.getExeFilePath() ).fileName() );
	ui -> qleBinaryPath -> setText( SETTINGS.getExeFilePath() );
	ui -> qwTabWelcome -> updateContent();
	ui -> qwTabFeatures -> updateContent();
	ui -> qwTabMod -> updateContent();
	ui -> qwTabVideo -> updateContent();
	ui -> qwTabAudioJoy -> updateContent();
	ui -> qwTabSpeech -> updateContent();
	ui -> qwTabNetwork -> updateContent();
	ui -> qwTabRegistry -> updateContent();
}

void MainWindow::flagsChanged()
{
	ui -> qwTabFeatures -> updateContent();
}

void MainWindow::registryChanged()
{
	ui -> qwTabRegistry -> updateContent();
}

void MainWindow::on_qpbBrowse_clicked()
{
	QString qstrPath = QFileDialog::getOpenFileName( this, "Select path to game executable", FILEPATHS.getGameDir() );
	if( !qstrPath.isEmpty() )
	{
		SETTINGS.setExeFilePath( qstrPath );
		updateContent();
	}
}

void MainWindow::on_qpbRun_clicked()
{
	on_qpbApply_clicked();

	/* if GAMEDIR is currentPath this does nothing, so its safe */
	System::chDir( FILEPATHS.getGameDir() );
	System::run( SETTINGS.getExeFilePath() );
}

void MainWindow::on_qpbOK_clicked()
{
	on_qpbApply_clicked();

	qApp->quit();
}

void MainWindow::on_qpbCancel_clicked()
{
	qApp->quit();
}

void MainWindow::on_qpbApply_clicked()
{
	SETTINGS.save();
	REGISTRY.save();
	registryChanged();
}
