#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <AL/alc.h>
#include <SDL/SDL.h>

#include "System.h"

#define BROWSERCMD	"browser"

/********************************** MISC **********************************/

bool System::run( const QString & qstrExec, const QStringList & qstrlParams, bool bWaitForEnd )
{
	if( qstrExec.isEmpty() )
		return false;

	if( QFileInfo( qstrExec ).isAbsolute() )
	{
		if( ! QFileInfo( qstrExec ).exists() )
			return false;
		if( ! QFileInfo( qstrExec ).isExecutable() )
			return false;
	}

	char *argv[ qstrlParams.size() + 2 ];
	int i, ret = 0;

	argv[ 0 ] = strdup( qstrExec.toAscii().data() );
	for( i = 0; i < qstrlParams.size(); i++ )
		argv[ i + 1 ] = strdup( qstrlParams.at( i ).toAscii().data() );
	argv[ i + 1 ] = NULL;

	pid_t pid = fork();
	if( pid == -1 )
		return false;
	if( pid == 0 )
		ret = execvp( argv[ 0 ], argv );
	if( ret == -1 )
		exit( 1 );
	if( bWaitForEnd )
		waitpid( pid, NULL, 0 );

	for( i = 0; argv[ i ] != NULL; i++ )
		delete argv[ i ];

	return true;
}

bool System::runBrowser( const QString & qstrUrl )
{
	if( qstrUrl.isEmpty() )
		return false;

	QString qstrExec = BROWSERCMD;
	QStringList qstrlParams;
	qstrlParams.append( qstrUrl );
	return System::run( qstrExec, qstrlParams );
}

int System::mSleep( int nMsec )
{
	return usleep( nMsec * 1000 );
}

QSize System::getSize()
{
	return QSize( 350, 550 );
}

/********************************** VIDEO **********************************/

QStringList System::enumerateResolutions()
{
	QStringList qstrl;

	if( SDL_InitSubSystem( SDL_INIT_VIDEO ) < 0 )
		return qstrl;

	/* Get available fullscreen/hardware modes */
	SDL_Rect** modes = SDL_ListModes( NULL, SDL_FULLSCREEN|SDL_HWSURFACE );

	if( modes == NULL )
		goto done;

	/* VERY strange case for me, what to do here? */
	//if( modes == (SDL_Rect**)-1 )
	//	goto done;

	// SDL seems to keep the modes sorted
	for( int i = 0; modes[ i ]; ++i )
	{
		if( modes[ i ]->w < 640 )
			continue;
		if( modes[ i ]->h < 480 )
			continue;

		qstrl.append( QString( "%1x%2" ).arg( modes[ i ]->w ).arg( modes[ i ]->h ) );
	}

done:
	SDL_QuitSubSystem( SDL_INIT_VIDEO );
	return qstrl;
}

/* nothing like that found in SDL, arbitraty will have to do */
QList< int > System::enumerateBPP()
{
	QList< int > ql;

	ql.append( 16 );
	ql.append( 32 );

	return ql;
}

/********************************** AUDIO **********************************/

QStringList System::enumerateAudioDevices()
{
	QStringList qstrl;

	/* 0 separated list of devices terminated with two 0 characters */
	const char * devices = ( const char* )alcGetString( NULL, ALC_DEVICE_SPECIFIER );

	if( devices == NULL )
		return qstrl;

	char * pos = ( char * )devices;

	while( *pos /*&& (strlen(pos) > 0 )*/ )
	{
		qstrl.append( QString( pos ) );
		pos += ( strlen( pos ) + 1 );
	}

	return qstrl;
}

QString System::getGenericDevice()
{
	return QString();
}

/********************************** INPUT **********************************/

QStringList System::enumerateJoystickDevices()
{
	QStringList qstrl;

	if( SDL_InitSubSystem( SDL_INIT_JOYSTICK ) < 0 )
		return qstrl;

	int nJoys = SDL_NumJoysticks();

	if( nJoys <= 0 )
		goto done;

	for( int i = 0; i < nJoys; i++ )
		qstrl.append( SDL_JoystickName( i ) );

done:
	SDL_QuitSubSystem( SDL_INIT_JOYSTICK );
	return qstrl;
}

void System::runCalibrateDialog( const int & nJoy )
{
	System::warning( QString( "Selected joystick is /dev/input/js%1, if you know any nice graphical joystick calibration utility for Linux please let me know." ).arg( nJoy ) );
}

/********************************** SPEECH **********************************/

void System::speechInit()
{
}

void System::speechDeinit()
{
}

bool System::speechReady()
{
	return false;
}

QStringList System::enumerateVoices()
{
	return QStringList();
}

void System::speechSetVoice( const int& )
{
}

void System::speechSetVolume( const int& )
{
}

void System::speechPlay( const QString& )
{

}

void System::speechStop()
{
}
